////////////////////////////////////////////////////////////////////////////////
//                       ZCommand 1.4b2 - Aston Plug-in
//                       by Zealot-6 (Travis Collins)
//                           tctx@earhtlink.net
//
////////////////////////////////////////////////////////////////////////////////
// Changelog
////////////////////////////////////////////////////////////////////////////////
//
//
// 1.4b2:  8/18/05
// Bug Fix: fixed a memory leak
//
// 1.4b1:  3/13/04
// Code Change: Made the history move recently used items to the top
// Code Change: Won't add items to the history if it was an invalid command
// Bug Fix: rearranged some code to make separating the arguments take place before converting special characters to hex
//
// 1.34b:  1/13/04
// Bug Fix: Corrected ZCommand sometimes disspearing if you have bevels turned off
//
// 1.33b: 12/31/03
// Bug Fix: Search now converts non alpha-numeric characters to their hex values
//
// 1.32b: 12/28/03
// Feature Update: CTRL+ENTER will now detect if you have already entered a
//                 domain extension.
//
// 1.31b: 12/27/03
// Bug Fix: Recycle was causing plugin to crash
//
// 1.3b: 12/25/03
// Bug Fix: Plugin was not initializing correctly if the expand was set to 0
//
// 1.2b: 12/24/03
// Made plugin a Child when not focused
// ZCommand will not lose focus after using calculator
//
// 1.1b: 12/08/03
// Bug Fix: fixed a small bug in AutoComplete
// Feature Added: Custom Message Handler to give ZCommand Focus
// Feature Added: Custom Message Handler to refresh Aliases
// Also created 2 stand alone exe's to send these messages
//
// 1.0: 12/07/03
// Public Release
// Bug Fix: focus issues
// Feature Added: customizable argument delimiter
//
// 0.62b: 12/06/03
// Bug Fix: After using the calculator, the result wouldn't clear when
//                clicking on zcommand again
// Feature Added: Help file
//
// 0.61b: 12/06/03
// Bug Fix: Disabled Alt+F4
// Changed most of the settings to be theme specific
// Made AutoComplete show newer entries first
// Feature Added: more flexible arguments to the search engines:
//                       <arg1>, <arg2>, <arg3+>, etc
// Feature Added: Border size (no effect if you have a bevel)
//
//
// 0.6b: 11/28/03
// Bug Fix: Resizing taskbar would make ZCommand jump to upper left corner
// Bug Fix: ZCommand wouldn't update after being moved
//               (ie: after a systray icon was added)
// Bug Fix: After all windows were minimized, ZCommand would expand.
//               Moved expanding code to LBUTTONUP instead of GETFOCUS
// Bug Fix: Fixed bevel option
// Feature Added: Ability to turn off AutoComplete
//
// 0.5b: 11/28/03
// Feature Added: AutoComplete
// Feature Added: Expand with Focus
// Feature Added: Ctrl+enter (automatically turns
//                            "yahoo" into "http://www.yahoo.com")
//
// 0.4b: 11/24/03
// Bug Fix: fixed Font bug
//
// 0.3b: 11/24/03
// Feature: Color added
// Feature: Bevel option
// Feature: ?site command added
//
// 0.2b: 11/20/03
// First Public Beta Release
///////////////////////////////////////////////////////////////////////////////
library Zcmd;

uses
  Classes,
  Windows,
  Messages,
  ShellAPI,
  sysutils,
  strutils,
  Registry,
  FNxxx in '..\FNxxx.pas',
  AstonSDK in '..\AstonSDK.pas',
  st_api in '..\st_api.pas';

{$IFDEF Debug}
{$E .plg}
{$ELSE}
{$I ..\plg.pas}
{$ENDIF}

{$IMAGEBASE $00400000}

type
  TSearchEngine = record
    Identifier: string;
    Command: string;
  end;

type
  TAlias = record
    Alias: string;
    Command: string;
  end;

const
  szClassName:PChar = 'ZCmd';
 //szHelp - what is shown in the "Properties" dialog
 szHelp:PChar = 'zCommand 1.4 b2';

  szSize = 'Size';
  szFont = 'Font';
  szTextColor = 'Text Color';
  szBackColor = 'Background Color';
  szBorder = 'Border';
  szBevel = 'Bevel';
  szDebug = 'Debug Mode';
  szDefText = 'Default Text';
  szPosition = 'Position';
  szPriority = 'Priority';
  szExpand = 'Expand With Focus';
  szAutoComplete = 'Use AutoComplete';
  szDefBrowser = 'Default Browser';
  szDefBrowserParms = 'Browser Command-line Parameters';
  szArgDelimit = 'Argument Delimiter';

  MAX_RUN_STRING = 1024;
  MAX_RUN_HISTORY = 10;

  cPositions = 1;
  vPositions:array[0..cPositions] of PChar=('Left (Top)', 'Right (Bottom)');

  //KeyCount - how many keys total
  KeyCount = 13;

  //Keys - the actual array of keys used in the plugin info
  Keys: array[1..KeyCount] of TKey = (
    (szKey:szSize; nType:KEY_INT;  szHint: szSize; vDefault: 10; res1:0; res2:0),
    (szKey:szTextColor; nType:KEY_COLOR;  szHint: szTextColor; vDefault: $000000; res1:0; res2:0),
    (szKey:szBackColor; nType:KEY_COLOR;  szHint: szBackColor; vDefault: $FFFFFF; res1:0; res2:0),
    (szKey:szFont; nType:KEY_FONT;  szHint: szFont; vDefault: 0; res1:0; res2:0),
    (szKey:szBorder; nType:KEY_SCROLLBAR;  szHint: szBorder; vDefault: 0; res1:0; res2:5),
    (szKey:szBevel; nType:KEY_BOOL;  szHint: szBevel; vDefault: 0; res1:0; res2:0),
    (szKey:szDebug; nType:KEY_BOOL or KEY_NOTHEME;  szHint: szDebug; vDefault: 0; res1:0; res2:0),
    (szKey:szDefText; nType:KEY_STRING;  szHint: szDefText; vDefault: 0; res1:0; res2:0),
    (szKey:szPosition; nType:KEY_LIST;  szHint: szPosition; vDefault: 0; res1:cPositions; res2:0),
    (szKey:szExpand; nType:KEY_SCROLLBAR;  szHint: szExpand; vDefault: 0; res1:0; res2:200),
    (szKey:szPriority; nType:KEY_SCROLLBAR;  szHint: szPriority; vDefault: 0; res1:0; res2:100),
    (szKey:szArgDelimit; nType:KEY_STRING or KEY_NOTHEME;  szHint: szArgDelimit; vDefault: 0; res1:0; res2:0),
    (szKey:szAutoComplete; nType:KEY_BOOL;  szHint: szAutoComplete; vDefault: 1; res1:0; res2:0)
  );


var

  PluginInfo:TPluginInfo;
  AData: TAstonData;
  ChildWindow:hWnd=0;
  ContainerWindow:hWnd=0;
  hText:hWnd=0;
  hInstContainer:cardinal;
  hInst:cardinal;
  ClientRC:TRect;
  brsh: HBRUSH;
  EditProcOld:Pointer;
  MomProcOld: Pointer;
  rChildWindow:TRect;

  setSize: integer;
  setTColor, setBackColor: Cardinal;
  setFont: TFont;
  setBorder: integer;
  setBevel: boolean;
  setDebug: boolean;
  setDefaultText: string;
  setPosition: integer;
  setPriority: integer;
  setExpand: integer;
  setDefaultBrowser: string;
  setDefaultBrowserParms: string;
  setAutoComplete: boolean;
  setArgDelimit: char;

  SearchEngineFile: string;
  AliasFile: string;
  HelpFile: string;
  AutoCompleteFile: string;
  DefaultSearchEngine: pchar;
  SearchEngineList:array of TSearchEngine;
  AliasList :array of TAlias;
  tslHistory: TStringList;  // For Autocomplete
  cleartext: string;  // When selecting the zcommand, if the text is this, then zcommand will be cleared
  bExpanded : boolean=False;

procedure AstonMB(sMessage: string);forward;
procedure DoneModule; stdcall;forward;
procedure InitGlobalModule(const AstonData:PAstonData); stdcall;forward; 

{----------------------------------------------------------------}
{----------------------------------------------------------------}

//////////////////////////////////////////////////////////////////
//   function txt                                               //
//                                                              //
//   formats text as lowercase,                                 //
//   and trims leading and trailing spaces.                     //
//   Used for comparissons                                      //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function txt(input: string): string;
begin
  result := lowercase(trim(input));
end;

//////////////////////////////////////////////////////////////////
//   function SearchEngineInit                                  //
//                                                              //
//   Builds the list of search engines                          //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function SearchEngineInit(): bool;
var
  tslFile: TStringList;
  i, i2: integer;

begin
  try
    tslFile := tStringList.Create;
    tslFile.LoadFromFile(SearchEngineFile);
  except  //  filenotfound
    result := False;
    exit;
  end;

  i2 := 0;
  SetLength(SearchEngineList, tslFile.Count);
  for i := 0 to tslFile.Count -1 do                                                                             
  begin
    if txt(copy(tslFile[i], 0, pos(' ', tslFile[i])-1)) = 'default' then
    begin
      DefaultSearchEngine := pchar(txt(copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1)));
    end
    else if copy(tslFile[i], 0, 1) <> ';' then
    begin
      SearchEngineList[i2].Identifier := txt(copy(tslFile[i], 1, pos(' ', tslFile[i])));
      SearchEngineList[i2].Command := copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1);
      i2  := i2 + 1;
    end;
  end;
  result := true;
end;

//////////////////////////////////////////////////////////////////
//   function AliasListInit                                     //
//                                                              //
//   Builds the list of Aliases                                 //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function AliasListInit(): bool;
var
  tslFile: TStringList;
  i, i2: integer;
begin
  try
    tslFile := tStringList.Create;
    tslFile.LoadFromFile(AliasFile);
  except
    //   filenotfound
    result := False;
    exit;
  end;

  i2 := 0;
  SetLength(AliasList, tslFile.Count);
  for i := 0 to tslFile.Count -1 do
  begin
    if copy(tslFile[i], 0, 1) <> ';' then
    begin
      AliasList[i2].Alias := txt(copy(tslFile[i], 1, pos(' ', tslFile[i])));
      AliasList[i2].Command := trim(copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1));
      i2  := i2 + 1;
    end;
  end;
  result := true;
end;

//////////////////////////////////////////////////////////////////
//   function GetSearchAddress                                  //
//                                                              //
//   Returns the search engine URL for the specified            //
//      search engine                                           //
//   If SearchEngineID is '' then returns the default engine    //
//////////////////////////////////////////////////////////////////
function GetSearchAddress(SearchEngineID:string): string;
var
  i: integer;
begin
  // if search engine is not specified, return URL for default search engine
  if strlen(pchar(SearchEngineID)) = 0 then
  begin
    try
     result := SearchEngineList[0].Command;
    except
     result := '';
    end;
  end

  // else, look up specified search engine
  else
  begin
    try
      for i := 0 to length(SearchEngineList) - 1 do
      begin
        if txt(SearchEngineID) = txt(SearchEngineList[i].Identifier) then
        begin
          result := SearchEngineList[i].Command;
          exit;
        end;
      end;
    except
      result := '';
    end;
  end;
end;

//////////////////////////////////////////////////////////////////
//   function CalcExpression                                    //
//                                                              //
//   Calculator... evaluates a string expression                //
//   Returns result of evaluation                               //
//////////////////////////////////////////////////////////////////
function CalcExpression (const expression : string) : extended;
var
  pos : Integer;
  ch : char;

  function GetChar : char;
  begin
    if pos < Length (expression) then
    begin
      Inc (pos);
      ch := expression [pos]
    end
    else ch := #0;
    result := ch
  end;

  function GetNonWhitespace : char;
  begin
    repeat
      GetChar
    until not (ch in [' ', #9]);
    result := ch
  end;

  function CalcAddition : extended; forward;

  function CalcNumber : extended;
  var
    s : string;
    gotPoint : boolean;
  begin
    gotPoint := False;
    while ch in ['0'..'9'] do
    begin
      s := s + ch;
      if (GetChar = '.') and not gotPoint then
      begin
        s := s + ch;
        GetChar;
        gotPoint := True
      end
    end;
    result := StrToFloat (s)
  end;

  function CalcExpressionInBrackets : double;
  begin
    result := CalcAddition;
    if ch = ')' then
      GetChar
    else
      raise Exception.Create ('Missing '')'' in expression')
  end;

  function CalcTerm : extended;
  begin
    case ch of
      '0'..'9' : result := CalcNumber;
      '(' : result := CalcExpressionInBrackets;
      else raise Exception.Create ('Syntax error in expression')
    end
  end;

  function CalcSignedTerm : extended;
  begin
    case GetNonWhitespace of
      '+' : result := CalcSignedTerm;
      '-' : result := -CalcSignedTerm;
      else result := CalcTerm
    end
  end;

  function CalcPower : extended;
  begin
    result := CalcSignedTerm;
    while ch = '^' do
      result := exp (ln (result) * CalcSignedTerm)
  end;

  function CalcMultiplication : extended;
  begin
    result := CalcPower;
    while ch in ['*', '/'] do
      case ch of
        '*' : result := result * CalcPower;
        '/' : result := result / CalcPower
      end
  end;

  function CalcAddition : extended;
  begin
    result := CalcMultiplication;
    while ch in ['+', '-'] do
      case ch of
        '+' : result := result + CalcMultiplication;
        '-' : result := result - CalcMultiplication
      end
  end;

begin
  try
    pos := 0;
    result := CalcAddition
  except
    result := -1;
  end;
end;

//////////////////////////////////////////////////////////////////
//   function GetArgs                                           //
//                                                              //
//   Parses a command line                                      //
//   Returns only the arguments in command line                 //
//////////////////////////////////////////////////////////////////
function GetArgs(cmdLine:string): string;
var
argstart: integer;
begin
    argstart := 0;
    cmdLine := pchar(trim(cmdLine));
    if copy(cmdline, 0 , 1) = '"' then
      argstart := posex('"', cmdLine, 2)
    else
      argstart := pos(' ', cmdLine);


    if argstart = 0 then  // No arguments
    begin
      result := '';
      exit;
    end
    else  // Arguments
    begin
      result := trim(copy(cmdLine, argstart + 1, strlen(pchar(cmdLine)) - argstart));
    end;
end;


//////////////////////////////////////////////////////////////////
//   function RemoveArgs                                        //
//                                                              //
//   Parses a command line                                      //
//   Returns only the command in command line                   //
//////////////////////////////////////////////////////////////////
function RemoveArgs(cmdLine:string): string;
var
  argstart: integer;
begin
  argstart := 0;
  cmdLine := pchar(trim(cmdLine));
  if copy(cmdline, 0 , 1) = '"' then
    argstart := posex('"', cmdLine, 2)
  else
    argstart := pos(' ', cmdLine);


  if argstart = 0 then  // No arguments
  begin
    result := cmdLine;
    exit;
    end
    else  // Arguments
    begin
      result := trim(copy(cmdLine, 1, argstart));
    end;
end;


//////////////////////////////////////////////////////////////////
//   function GetAlias                                          //
//                                                              //
//   Searched alias array for a valid alias.                    //
//   Returns the command if sucessful,                          //
//   Returns original text if unsucessful                       //
//////////////////////////////////////////////////////////////////
function GetAlias(alias: string): string;
var
  i: integer;
begin
  alias := txt(alias);
  for i := 0 to Length(AliasList) - 1 do
  begin
    if alias = AliasList[i].Alias then
    begin
      result := AliasList[i].Command;
      exit;
    end;
  end;
  result := alias;
end;


//////////////////////////////////////////////////////////////////
//   function ExecCommand                                       //
//                                                              //
//   Executes a command line                                    //
//   if command line begins with ., treats it as a command      //
//   if command line begins with ?, launches an internet search //
//   if command line begins with =, evaluate expression         //
//   otherwise, launch program or website                       //
//////////////////////////////////////////////////////////////////
function ExecCommand(cmdLine: string): boolean;
var
  cmd: string;
  args: string;
  res: pchar;
  seID: string;
  shellret: integer;
  SLarg: TStringList;
  stemp: string;
  i: integer;
  chrvalue: integer;
begin
  if lstrlen(pchar(cmdLine))<1 then exit;

  // Help
  if (copy(txt(cmdline), 1, 4) = 'help') or (copy(txt(cmdline), 1, 4) = 'rtfm')
   then
  begin
    ShellExecute(0, 'OPEN', pchar(HelpFile), '', nil, SW_SHOWDEFAULT);
  end

  // Internet Search
  else if copy(cmdline, 0 , 1) = '?' then
  begin
    ////////////////////////
    // Default Search Engine
    ////////////////////////
    if copy(cmdLine, 2, 1) = ' ' then
    begin
      cmd := GetSearchAddress('');
      if strlen(pchar(cmd)) = 0 then
      begin
        AstonMB('Error: No Default Search Engine');
        result := False;
        exit;
      end;
    end

    //////////////////////////
    // Specify a Search Engine
    //////////////////////////
    else
    begin
      seID := copy(cmdLine, 2, pos(' ', cmdLine)-2);
      seID := trim(seID);
      cmd := GetSearchAddress(seID);
      if strlen(pchar(cmd)) = 0 then
      begin
        AstonMB('Error: Unknown Search Engine: ' + seID);
        result := False;
        exit;
      end;
    end;

    // Set args string to everything after the first space
    args := trim(copy(cmdLine, pos(' ', cmdLine), strlen(pchar(cmdLine)) - pos(' ', cmdLine)+1));

    stemp := args;
    while Pos(setArgDelimit, stemp)>0 do stemp[Pos(setArgDelimit, stemp)]:=#10;
    stemp := adjustlinebreaks(stemp);
    SLarg := TStringList.Create;
    SLarg.Text := stemp;

    // Replace all non alpha-numeric characters with their hex values
    for i := 0 to 31 do
      args := stringreplace(args, chr(i), '%' + inttohex(i, 2), [rfReplaceAll]);

    for i := 33 to 47 do
      args := stringreplace(args, chr(i), '%' + inttohex(i, 2), [rfReplaceAll]);

    for i := 58 to 64 do
      args := stringreplace(args, chr(i), '%' + inttohex(i, 2), [rfReplaceAll]);

    for i := 91 to 96 do
      args := stringreplace(args, chr(i), '%' + inttohex(i, 2), [rfReplaceAll]);

    for i := 123 to 255 do
      args := stringreplace(args, chr(i), '%' + inttohex(i, 2), [rfReplaceAll]);

    // Replace all spaces with +'s
    args := stringreplace(args, ' ', '+', [rfReplaceAll]);

{
    for i := 0 to strlen(pchar(args)) do
    begin
      chrvalue := ord(args[i]);
      if (chrvalue < 32) or
      ((chrvalue > 32) and (chrvalue < 48)) or
      ((chrvalue > 57) and (chrvalue < 65)) or
      ((chrvalue > 91) and (chrvalue < 97)) or
      (chrvalue > 122)
      then
        args[i] := ;
    end;
}



    try
      cmd := stringreplace(cmd, '<args>', args, [rfReplaceAll]);
    except end;

    for i := 0 to 4 do
    begin
      try
        cmd := stringreplace(cmd, '<arg' + inttostr(i + 1) + '+>', copy(args, pos(SLarg[i], args), strlen(pchar(args))), [rfReplaceAll]);
      except
        cmd := stringreplace(cmd, '<arg' + inttostr(i +1) + '+>', '', [rfReplaceAll]);
      end;

      try
        cmd := stringreplace(cmd, '<arg' + inttostr(i+1) + '>', SLarg[i], [rfReplaceAll]);
      except
        cmd := stringreplace(cmd, '<arg' + inttostr(i+1) + '>', '', [rfReplaceAll]);
      end;
    end;

    cmd := trim(cmd);
    SLarg.Free;

    if setDebug then
      AstonMB('-Debug-'#13#10+'Command Line: ' + cmdLine + #13#10 + 'Cmd: ' + cmd);

    shellret := ShellExecute(0,'open',pchar(cmd),'',nil,SW_SHOWDEFAULT);
    if shellret < 32 then
    begin
      result := False;
      AstonMB('Error occured while launching internet search');
    end
    else
      Result := True;
  end

  // Calculator
  else if copy(cmdline, 0 , 1) = '=' then
  begin
    args := pchar(copy(cmdLine, 2, strlen(pchar(cmdLine)) - 1));
    res := pchar(FloatToStrF(CalcExpression(args), ffGeneral	, 15, 14));
    SetWindowText(htext, res);
    ClearText := res;
    exit;
  end

  // Either an Alias or an executable
  else
  begin
    // Get the typed arguments
    args := GetArgs(cmdLine);
    // Get the typed command
    cmd := RemoveArgs(cmdLine);
    // was the typed command an alias?
    cmd := GetAlias(cmd);
    // add alias arguments to typed arguments
    args := GetArgs(cmd) + args;
    // strip arguments from the cmd
    cmd := RemoveArgs(cmd);
    // allow for 1 level of recursion
    cmd := GetAlias(cmd);

    if setDebug then
      AstonMB('-Debug-'#13#10+'Command Line: ' + cmdLine + #13#10 + 'Cmd: ' + cmd + #13#10 + 'Args: ' + args);

    shellret := ShellExecute(0,'open',pchar(cmd),pchar(args),nil,SW_SHOWDEFAULT);

    if shellret < 32 then
    begin
      result := False;
      AstonMB('Error: ' + SysErrorMessage(GetLastError));
    end
    else
      Result := True;
  end;
  SetWindowText(hText,pchar(setDefaultText));
end;

//////////////////////////////////////////////////////////////////
//   procedure AutoCompleteInit                                 //
//                                                              //
//   Reads the history file into a string list                  //
//////////////////////////////////////////////////////////////////
procedure AutoCompleteInit();
begin
  tslHistory := TStringlist.Create;
  try
    tslHistory.LoadFromFile(AutoCompleteFile);
  except
    tslHistory.SaveToFile(AutoCompleteFile);
  end;
end;


//////////////////////////////////////////////////////////////////
//   function AutoComplete                                      //
//                                                              //
//   Arguments:                                                 //
//      hText - HWND, Handle to the textbox that will receive   //
//                  the resultant AutoCompleted string          //
//      sText - STRING: string to autocomplete                  //
//                                                              //
//   Returns True upon sucess, false upon not finding a match   //
//////////////////////////////////////////////////////////////////
function AutoComplete(hText:HWND; sText: string): boolean;
var
  i: integer;
begin
  result := FALSE;
  if strlen(pchar(stext)) > 0 then
  begin
    for i := 0 to tslHistory.Count - 1 do
    begin
      if copy(lowercase(tslHistory[i]), 0, strlen(pchar(stext))) = lowercase(stext) then
      begin
        SetWindowText(htext, pchar(tslHistory[i]));
        SendMessage(htext, EM_SETSEL, strlen(pchar(tslHistory[i])), strlen(pchar(stext)));
        result := TRUE;
        exit;
      end;
    end;
  end;
end;

//////////////////////////////////////////////////////////////////
//   function AutoCompleteAdd                                   //
//                                                              //
//   Arguments:                                                 //
//      sText - STRING: string to add to history                //
//                                                              //
//   Returns True upon sucessful add,                           //
//           false upon finding a match                         //
//////////////////////////////////////////////////////////////////
procedure AutoCompleteAdd(sText: string);
var
  i: integer;
begin
  if strlen(pchar(stext)) > 0 then
  begin
    for i := 0 to tslHistory.Count - 1 do
    begin
      if copy(txt(tslHistory[i]), 0, strlen(pchar(stext))) = txt(stext) then
      begin
        tslHistory.Move(i, 0);
        tslHistory.SaveToFile(AutoCompleteFile);
        exit;
      end;
    end;
    // Still here?  must be a new entry
    tslHistory.Add(sText);
    tslHistory.Move(tslHistory.Count-1, 0);  // Move new entry to the top
    tslHistory.SaveToFile(AutoCompleteFile);
//    AutoCompleteInit();
  end;
end;


//////////////////////////////////////////////////////////////////
//   function MomProc                                           //
//                                                              //
//   Subclass parent of textbox                                 //
//   to handle changing colors of the textbox                   //
//////////////////////////////////////////////////////////////////
function MomProc(Window: HWnd; Message:UINT; WParam: WParam; LParam: LParam): Longint;  stdcall;
begin
  result := 0;
   case Message of
    WM_CTLCOLOREDIT:
    begin
      if bExpanded then
      begin
        SetBkColor(wparam, setBackColor);
        DeleteObject(brsh);
        brsh := CreateSolidBrush(setBackColor);
        SetTextColor(wParam, settcolor);
        result := brsh;
        exit;
      end
      else
      begin
        SetBkColor(wparam, setBackColor);
        DeleteObject(brsh);
        brsh := CreateSolidBrush(setBackColor);
        SetTextColor(wParam, settcolor);
        result := brsh;
        exit;
      end;
    end;
  end;
  result:= CallWindowProc(MomProcOld, Window,Message,wParam,lParam);
end;



//////////////////////////////////////////////////////////////////
//   procedure Expand                                           //
//                                                              //
//   Expands or colapses ZCommand                               //
//   depending on wether state is true or false                 //
//////////////////////////////////////////////////////////////////
procedure Expand(expanded: boolean);
var
  WindowStyle: integer;
begin
    if expanded = true then
    begin

      WindowStyle := GetWindowLong(ContainerWindow, GWL_STYLE);
      WindowStyle := WindowStyle and (not WS_CHILD);
      WindowStyle := WindowStyle or WS_POPUP;
      SetWindowLong(ContainerWindow, GWL_STYLE, WindowStyle);

      ShowWindow(ContainerWindow, SW_HIDE);
      SetParent(ContainerWindow, 0);

      case AData.Ring1.TaskbarPos of
      0: // Left Taskbar
      begin
        SetWindowPos(ContainerWindow, HWND_TOPMOST, rChildWindow.Left, rChildWindow.Top, rChildWindow.Right - rChildWindow.Left + setExpand, rChildWindow.Bottom - rChildWindow.Top, 0);
        MoveWindow(hText, setBorder, setBorder, rChildWindow.Right - rChildWindow.Left + setExpand - (setBorder*2), rChildWindow.Bottom - rChildWindow.Top - (setBorder*2), True);
      end;
      1,3: // Top or Bottom Taskbar
      begin
        if setPosition = 0 then // Left ZCommand
        begin
          SetWindowPos(ContainerWindow, HWND_TOPMOST, rChildWindow.Left, rChildWindow.Top, rChildWindow.Right - rChildWindow.Left + setExpand, rChildWindow.Bottom - rChildWindow.Top, 0);
          MoveWindow(hText, setBorder, setBorder, rChildWindow.Right - rChildWindow.Left + setExpand - (setBorder*2), rChildWindow.Bottom - rChildWindow.Top - (setBorder*2), True);
        end
        else                    // Right ZCommand
        begin
          SetWindowPos(ContainerWindow, HWND_TOPMOST, rChildWindow.Left - setExpand, rChildWindow.Top, rChildWindow.Right - rChildWindow.Left + setExpand, rChildWindow.Bottom - rChildWindow.Top, 0);
          MoveWindow(hText, setBorder, setBorder, rChildWindow.Right - rChildWindow.Left - (setBorder*2) + setExpand, rChildWindow.Bottom - rChildWindow.Top - (setBorder*2), True);
        end;
      end;
      2: // Right Taskbar
      begin
        SetWindowPos(ContainerWindow, HWND_TOPMOST, rChildWindow.Left - setExpand, rChildWindow.Top, rChildWindow.Right - rChildWindow.Left + setExpand, rChildWindow.Bottom - rChildWindow.Top, 0);
        MoveWindow(hText, setBorder, setBorder, rChildWindow.Right - rChildWindow.Left - (setBorder*2) + setExpand, rChildWindow.Bottom - rChildWindow.Top - (setBorder*2), True);
      end;
      end;
      ShowWindow(ContainerWindow, SW_SHOW);
    end
    else
    begin
      WindowStyle := GetWindowLong(ContainerWindow, GWL_STYLE);
      WindowStyle := WindowStyle and (not WS_POPUP);
      WindowStyle := WindowStyle or WS_CHILD;
      SetWindowLong(ContainerWindow, GWL_STYLE, WindowStyle);

      ShowWindow(ContainerWindow, SW_HIDE);
      SetParent(ContainerWindow, ChildWindow);
      SetWindowPos(ContainerWindow, 0, 0, 0, rChildWindow.Right - rChildWindow.Left, rChildWindow.Bottom - rChildWindow.Top, 0);
      SetWindowPos(hText, 0, setBorder, setBorder, rChildWindow.Right - rChildWindow.Left - (setBorder*2), rChildWindow.Bottom - rChildWindow.Top - (setBorder*2), 0);
      ShowWindow(ContainerWindow, SW_SHOW);

    end;
    bExpanded := expanded;
    if expanded = true then setfocus(htext);
end;


//////////////////////////////////////////////////////////////////
//   function EditProc                                          //
//                                                              //
//   Subclass Proc for textbox                                  //
//   to handle capturing return key                             //
//////////////////////////////////////////////////////////////////
function EditProc(Handle: HWND;  msg: UINT;  wParam: WPARAM;  lParam: LPARAM): integer; stdcall;
const
 SC_MenuItem1 = WM_USER + 1;
 SC_MenuItem2 = SC_MenuItem1 + 1;
 SC_MenuItem3 = SC_MenuItem2 + 1;
 SC_MenuItem4 = SC_MenuItem3 + 1;
 SC_MenuItem5 = SC_MenuItem4 + 1;
 SC_MenuItem6 = SC_MenuItem5 + 1;
 SC_MenuItem7 = SC_MenuItem6 + 1;
 SC_MenuItem8 = SC_MenuItem7 + 1;
 SC_MenuItem9 = SC_MenuItem8 + 1;
 SC_MenuItem10 = SC_MenuItem9 + 1;
 SC_MenuItem11 = SC_MenuItem10 + 1;
 SC_MenuItem12 = SC_MenuItem11 + 1;
var
  buf: array [0..MAX_RUN_STRING] of char;
  buftmp: pchar;
  szSetup: array[0..MAX_PATH+1] of char;
  PopMenu : HMenu;
  r: TRect;
  bRet: boolean;
begin

  case msg of
    WM_CONTEXTMENU:
    begin
    PopMenu := CreatePopupMenu;

    // Duplicate the Default Menu
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem1, '&Undo');
    AppendMenu(PopMenu, MF_SEPARATOR, 0, '');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem2, 'Cu&t');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem3, '&Copy');
    AppendMenua(PopMenu, MF_STRING, SC_MenuItem4, '&Paste');
    AppendMenu(PopMenu, MF_SEPARATOR, 0, '');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem5, 'Select &All');
    AppendMenu(PopMenu, MF_SEPARATOR, 0, '');

    {add our menu}
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem6, 'Edit Search Engine List');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem7, 'Edit Aliases');
    if setAutoComplete then
    begin
      AppendMenu(PopMenu, MF_STRING, SC_MenuItem8, 'Edit History');
      AppendMenu(PopMenu, MF_STRING, SC_MenuItem9, 'Clear History');
    end;
    AppendMenu(PopMenu, MF_SEPARATOR, 0, '');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem10, 'Setup ZCommand');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem11, 'Recycle ZCommand');
    AppendMenu(PopMenu, MF_STRING, SC_MenuItem12, 'Help');



    TrackPopupMenu(PopMenu, TPM_LEFTALIGN, LOWORD(lParam), HIWORD(lParam), 0, Handle, nil);
    DestroyMenu(PopMenu);

      result := 0;
      exit;
    end;
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        SC_MenuItem1:   // Undo
          SendMessage(Handle, WM_UNDO, 0,0);
        SC_MenuItem2:   // Cut
          SendMessage(Handle, WM_CUT, 0,0);
        SC_MenuItem3:   // Copy
          SendMessage(Handle, WM_COPY, 0,0);
        SC_MenuItem4:   // Paste
          SendMessage(Handle, WM_PASTE, 0,0);
        SC_MenuItem5:   // Select All
          SendMessage(Handle, EM_SETSEL, 0, -1);
        SC_MenuItem6:   // Edit Search Engine List
        begin
          ShellExecute(0, 'OPEN', pchar(SearchEngineFile), '', nil, SW_SHOWDEFAULT);
        end;
        SC_MenuItem7:   // Edit Aliases
          ShellExecute(0, 'OPEN', pchar(AliasFile), '', nil, SW_SHOWDEFAULT);
        SC_MenuItem8:   // Edit History
          ShellExecute(0, 'OPEN', pchar(AutoCOmpleteFile), '', nil, SW_SHOWDEFAULT);
        SC_MenuItem9:   // Clear History
        begin
          if MessageBox(0, pchar('Clear AutoComplete History?'), pchar('Aston ZCommand'), MB_YESNO or MB_TOPMOST) = IDYES then
          begin
            tslHistory.Clear;
            tslHistory.SaveToFile(AutoCompleteFile);
          end;
        end;
        SC_MenuItem10:   // Setup ZCommand
          begin
            szSetup[0] := '/';
            lstrcpy(@szSetup[1], Adata.Node.Path);
            SendMessage(Adata.MainWindow, CM_SETUPASTON, 0, integer(@szSetup[0]));
          end;
        SC_MenuItem11:   // Recycle ZCommand
          begin
            DoneModule();
            InitGlobalModule(@AData);
          end;
        SC_MenuItem12:   // Help
          ShellExecute(0, 'OPEN', pchar(HelpFile), '', nil, SW_SHOWDEFAULT);
      end;
    end;
    WM_LBUTTONUP, WM_RBUTTONUP, WM_MBUTTONUP, WM_NCLBUTTONUP, WM_NCRBUTTONUP, WM_NCMBUTTONUP, WM_KEYDOWN:
    begin
        GetWindowText(hText, buf, sizeof(buf));
        if (txt(buf) = txt(setDefaultText)) or (txt(buf) = txt(cleartext)) then
          SetWindowText(hText, pchar(''));
          GetWindowRect(ChildWindow, rChildWindow);

        if not bExpanded then Expand(true);
    end;
    WM_KILLFOCUS:
    begin
      GetWindowText(Handle, buf, sizeof(buf));
      if bExpanded and (txt(buf) = '') then
        SetWindowText(Handle, pchar(setDefaultText));


        if bExpanded then Expand(false);
    end;
    WM_SYSCOMMAND:
    begin
      result := 1;
      exit;
    end;
    WM_CHAR:
    begin
      if wParam = $0A then       // CTRL + ENTER
      begin
        GetWindowText(Handle, buf, sizeof(buf));
        if pos('.', buf)>0 then
          buftmp := pchar('http://www.' + buf)
        else
          buftmp := pchar('http://www.' + buf + '.com');
        bRet := ExecCommand(buftmp);
        if setAutoComplete and bRet then AutoCompleteAdd(buf);
        result := 0;
        exit;
      end;
      if wParam=VK_RETURN then
      begin
        GetWindowText(Handle, buf, sizeof(buf));
        if buf[0] <> '=' then
          SetFocus(AData.MainWindow);
        bRet := ExecCommand(buf);
        if setAutoComplete and bRet then AutoCompleteAdd(buf);
        result := 0;
        exit;
      end
      else if wParam=VK_ESCAPE then
      begin
        SetWindowText(Handle, pchar(setDefaultText));
        SetFocus(AData.MainWindow);
      end
      else if (wParam <> VK_DELETE) and (wParam <> VK_BACK) then // Autocomplete
      begin
        // Go ahead and type the character
        CallWindowProc(EditProcOld,Handle,msg,wParam,lParam);

        // Auto Complete
        if setAutoComplete then
        begin
          GetWindowText(Handle, buf, sizeof(buf));
          AutoComplete(Handle, buf);
        end;
        exit;
      end;
    end;
  end;{case}
  begin
    result:= CallWindowProc(EditProcOld,Handle,msg,wParam,lParam);
    exit;
  end;
end;

{----------------------------------------------------------------}
{----------------------------------------------------------------}
function WindowProc(Window: HWnd; Message:UINT; WParam: WParam; LParam: LParam): Longint;  stdcall;
const
 CM_SETFOCUS = WM_USER+5;
 CM_ADDALIAS = CM_SETFOCUS+1;
var
  DC: hDC;
  ps: TPaintStruct;
  r2:Trect;
  uFlags:UINT;
  buf: array [0..MAX_RUN_STRING] of char;
begin
  result := 0;
  case Message of
    CM_SETFOCUS:
    begin
      SetForegroundWindow(htext);
      SetFocus(htext);
      GetWindowText(htext, buf, sizeof(buf));
      if (txt(buf) = txt(setDefaultText)) or (txt(buf) = txt(cleartext)) then
        SetWindowText(htext, pchar(''));
      if not bExpanded then Expand(true);
    end;
    CM_ADDALIAS:
    begin
      AliasListInit();
    end;
    WM_CREATE:
    begin
      htext := 0;
    end;
    WM_CTLCOLOREDIT:
    begin
      SetBkColor(wparam, setBackColor);
        DeleteObject(brsh);
      brsh := CreateSolidBrush(setBackColor);
      SetTextColor(wParam, settcolor);
      result := brsh;
      exit;
    end;
    WM_PAINT:
    begin
      DC:=BeginPaint(Window, ps);
      GetClientRect(Window, ClientRC);
      SetRect(r2, 0,0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN));
      uFlags:= DPEx_CORRECTBRUSH;

      if htext = 0 then
      begin
        GetWindowRect(ChildWindow, rChildWindow);

        ContainerWindow := CreateWindowEx(0, szClassName, 'ZCommand', WS_POPUP or WS_EX_TOOLWINDOW, 0,0, ClientRC.Right, ClientRC.Bottom, Window, 0, hInstContainer, 0);
        if setbevel then
	          htext := CreateWindowEx(WS_EX_CLIENTEDGE,'EDIT','ZCommand',  WS_CHILD or ES_AUTOHSCROLL or WS_EX_TOOLWINDOW,setBorder,setBorder,ClientRC.Right - (setBorder*2),ClientRC.Bottom - (setBorder*2),ContainerWindow,0,hInst,0)
        else
           htext := CreateWindowEx(0,'EDIT','ZCommand',  WS_CHILD or ES_AUTOHSCROLL or WS_EX_TOOLWINDOW ,setBorder,setBorder,ClientRC.Right - (setBorder*2),ClientRC.Bottom - (setBorder*2),ContainerWindow,0,hInst,0);

          MomProcOld := pointer(SetWindowLong(getparent(hText), GWL_WNDPROC, LongInt(@MomProc)));
          SetWindowPos(hText,HWND_TOPMOST, setBorder, setBorder, ClientRC.Right - ClientRC.Left - (setBorder * 2), ClientRC.Bottom - ClientRC.Top - (setBorder * 2), 0);
          ShowWindow(ContainerWindow, SW_SHOW);
          ShowWindow(hText, SW_SHOW);
          SetFocus(AData.MainWindow);
          SelectObject(htext, setFont.Handle);
          SendMessage(htext, WM_SETFONT, setFont.Handle, 1);
          EditProcOld := pointer(SetWindowLong(hText, GWL_WNDPROC, LongInt(@EditProc)));
          SetWindowText(hText,pchar(setDefaultText));
          Expand(false);
      end;
      EndPaint(Window, ps);
      ReleaseDC(Window, dc);
      exit;
    end;
    WM_DROPFILES:
    begin
      ///////////////////////////
      // TODO: add drag n drop //
      ///////////////////////////
    end;
    WM_LBUTTONUP, WM_RBUTTONUP, WM_MBUTTONUP, WM_KEYDOWN:
    begin
      SetFocus(htext);
      GetWindowText(htext, buf, sizeof(buf));
      if (txt(buf) = txt(setDefaultText)) or (txt(buf) = txt(cleartext)) then
        SetWindowText(hText, pchar(''));
      if not bExpanded then Expand(true);
    end;
  end;
  result := DefWindowProc(Window, Message, WParam, LParam);
end;

{----------------------------------------------------------------}
procedure ReadConfig;
var
  CFG:TCFG;
  temp: array[0..MAX_PATH] of char;
  temp2: array[0..MAX_PATH] of char;
begin
  OpenCFGr(AData.PluginsCFG, @CFG);

  FindSection(szClassName, 0, @CFG);
  setSize:=GetCFGInt(szSize, 150, @CFG);

  GetCFGFont(szFont, @setFont, @CFG);
  GetFontHandle(@setFont, GetStockObject(ANSI_VAR_FONT));

  setTColor := GetCFGHex(szTextColor, $000000, @CFG);
  setBackColor := GetCFGHex(szBackColor, $FFFFFF, @CFG);

  setBevel := GetCFGBool(szBevel, FALSE, @CFG);
  setBorder:=GetCFGInt(szBorder, 0, @CFG);
    
  setDebug := GetCFGBool(szDebug, FALSE, @CFG);
  setAutoComplete := GetCFGBool(szAutoComplete, TRUE, @CFG);
  
  GetCFGString(szDefText, '', temp, SizeOf(temp), @CFG);
  setDefaultText := pchar(trim(temp));

  GetCFGString(szDefBrowser, '', temp, SizeOf(temp), @CFG);
  setDefaultBrowser := pchar(trim(temp));

  GetCFGString(szDefBrowserParms, '', temp, SizeOf(temp), @CFG);
  setDefaultBrowserParms := pchar(trim(temp));

  GetCFGString(szArgDelimit, '/', temp2, SizeOf(temp2), @CFG);
  setArgDelimit := temp2[0];

  setPosition := GetCFGInt(szPosition, 0, @CFG);
  setPriority := GetCFGInt(szPriority, 0, @CFG);
  setExpand := GetCFGInt(szExpand, 0, @CFG);
  CloseCFGr(@CFG);
end;

{----------------------------------------------------------------}

procedure InitGlobalModule(const AstonData:PAstonData);stdcall;
var
  WindowClass:TWndClass;
  DockInfo: TDockInfo;
  sPath: array[0..MAX_PATH] of char;
begin
  RtlMoveMemory(@AData, AstonData, SizeOf(TAstonData));

  RtlZeroMemory(@WindowClass, sizeOf(TWndClass));
  WindowClass.style:=CS_HREDRAW or CS_VREDRAW;
  WindowClass.hCursor := LoadCursor(0, IDC_ARROW);
  WindowClass.hbrBackground:=COLOR_BTNFACE+1;
  WindowClass.hInstance := HInstance;
  WindowClass.lpfnWndProc:=@WindowProc;
  WindowClass.lpszClassName:= szClassName;
  if RegisterClass(WindowClass) = 0 then exit;


  ReadConfig;

  ChildWindow:= CreateWindowEx(0,
    szClassName,
    nil,
    WS_VISIBLE or WS_CHILD or WS_CLIPSIBLINGS or WS_CLIPCHILDREN or WS_EX_ACCEPTFILES,
    0, 0,
    0, 0,
    AData.Taskbar,
    0,
    hInstance,
    nil);
  AData.Node.Wnd:= ChildWindow;
  DragAcceptFiles(ChildWindow, TRUE);
  with DockInfo do
  begin
    Wnd:=ChildWindow;
    Parent:= AData.Taskbar;

    // Top or Bottom
    if (AData.Ring1.TaskbarPos = 1) or (AData.Ring1.TaskbarPos = 3) then
      SetRect(rc_Client, 0,0, setsize, AData.Ring1.ButtonHeight)
    // Left or Right
    else
      if setsize > 0 then
        SetRect(rc_Client, 0,0, AData.ring1.BarSize, setsize)
      else
        SetRect(rc_Client, 0,0, AData.ring1.BarSize, AData.ring1.ButtonHeight);
    Align:= setPosition;
    Vert:=false;
    Flat:=false;
    Priority:=setPriority;
  end;
  SendMessage(AData.MainWindow, CM_DOCK, 0, longint(@DockInfo));
  GetModuleFileName(hinstance, sPath, sizeof(sPath));
  SearchEngineFile := extractfilepath(spath) + 'zcmdengines.txt';
  AliasFile := extractfilepath(spath) + 'zcmdaliases.txt';
  AutoCompleteFile := extractfilepath(spath) + 'zcmdhistory.txt';
  HelpFile := ExtractFilePath(spath) + 'ZCommand.hlp';
  SearchEngineInit();
  AliasListInit();
  AutoCompleteInit();
end;

function GetPluginInfo:PPluginInfo;stdcall;
begin
  PluginInfo.cbSize:= SizeOf(TPluginInfo);
  PluginInfo.szName:= szClassName;
  PluginInfo.szDescription:= szhelp;
  PluginInfo.szSection:= szClassName;
  PluginInfo.nKeyCount:= KeyCount;
  PluginInfo.Keys:= @Keys;
  Keys[9].res2:=integer(@vPositions);
  result:= @PluginInfo;
end;

{----------------------------------------------------------------}

procedure DoneModule; stdcall;
begin
  if htext <> 0 then
  begin
    Expand(false);
    SetWindowLong(GetParent(hText), GWL_WNDPROC, LongInt(MomProcOld));
    SetWindowLong(hText, GWL_WNDPROC, LongInt(EditProcOld));
    DestroyWindow(htext);
    htext := 0;
  end;

  if ChildWindow<>0 then
  begin
    DestroyWindow(ChildWindow);
    ChildWindow:=0;
  end;
  UnregisterClass(szClassName, hInstance);
end;

{----------------------------------------------------------------}

//////////////////////////////////////////////////////////////////
//   procedure AstonMB                                          //
//                                                              //
//   Shows Message Box                                          //
//////////////////////////////////////////////////////////////////
procedure AstonMB(sMessage: string);
// Modal message box
begin
  SendMessage(AData.MainWindow, CM_ENTERMODAL, 0, 0);
  MessageBox(0, pchar(sMessage), pchar('Aston ZCommand'), MB_OK or MB_TOPMOST);
  SendMessage(AData.MainWindow, CM_LEAVEMODAL, 0, 0);
end;


exports
  GetPluginInfo,
  InitGlobalModule,
  DoneModule;
end.
